# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'optimization_extra_data_add_protocol_map_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_AddProtocolMapDialog(object):
    def setupUi(self, AddProtocolMapDialog):
        AddProtocolMapDialog.setObjectName("AddProtocolMapDialog")
        AddProtocolMapDialog.resize(719, 190)
        self.verticalLayout = QtWidgets.QVBoxLayout(AddProtocolMapDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label_3 = QtWidgets.QLabel(AddProtocolMapDialog)
        font = QtGui.QFont()
        font.setPointSize(14)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_3.addWidget(self.label_3)
        self.label_4 = QtWidgets.QLabel(AddProtocolMapDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_3.addWidget(self.label_4)
        self.verticalLayout.addLayout(self.verticalLayout_3)
        self.line = QtWidgets.QFrame(AddProtocolMapDialog)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label_10 = QtWidgets.QLabel(AddProtocolMapDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_10.setFont(font)
        self.label_10.setObjectName("label_10")
        self.gridLayout.addWidget(self.label_10, 0, 2, 1, 1)
        self.label = QtWidgets.QLabel(AddProtocolMapDialog)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.valueInput = QtWidgets.QLineEdit(AddProtocolMapDialog)
        self.valueInput.setEnabled(True)
        self.valueInput.setObjectName("valueInput")
        self.horizontalLayout.addWidget(self.valueInput)
        self.fileBrowse = QtWidgets.QPushButton(AddProtocolMapDialog)
        self.fileBrowse.setEnabled(True)
        self.fileBrowse.setObjectName("fileBrowse")
        self.horizontalLayout.addWidget(self.fileBrowse)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 1, 1, 1)
        self.label_12 = QtWidgets.QLabel(AddProtocolMapDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.label_12.setFont(font)
        self.label_12.setObjectName("label_12")
        self.gridLayout.addWidget(self.label_12, 1, 2, 1, 1)
        self.mapNameInput = QtWidgets.QLineEdit(AddProtocolMapDialog)
        self.mapNameInput.setObjectName("mapNameInput")
        self.gridLayout.addWidget(self.mapNameInput, 0, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(AddProtocolMapDialog)
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 1, 0, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.line_3 = QtWidgets.QFrame(AddProtocolMapDialog)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.verticalLayout.addWidget(self.line_3)
        self.buttonBox = QtWidgets.QDialogButtonBox(AddProtocolMapDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(AddProtocolMapDialog)
        self.buttonBox.accepted.connect(AddProtocolMapDialog.accept)
        self.buttonBox.rejected.connect(AddProtocolMapDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(AddProtocolMapDialog)
        AddProtocolMapDialog.setTabOrder(self.mapNameInput, self.valueInput)
        AddProtocolMapDialog.setTabOrder(self.valueInput, self.fileBrowse)

    def retranslateUi(self, AddProtocolMapDialog):
        _translate = QtCore.QCoreApplication.translate
        AddProtocolMapDialog.setWindowTitle(_translate("AddProtocolMapDialog", "Add map"))
        self.label_3.setText(_translate("AddProtocolMapDialog", "Add protocol map"))
        self.label_4.setText(_translate("AddProtocolMapDialog", "Add a protocol map to the input data"))
        self.label_10.setText(_translate("AddProtocolMapDialog", "(The map name matching a protocol parameter of the model)"))
        self.label.setText(_translate("AddProtocolMapDialog", "Map name: "))
        self.fileBrowse.setText(_translate("AddProtocolMapDialog", "File browser"))
        self.label_12.setText(_translate("AddProtocolMapDialog", "(Either a nifti file or a scalar value)"))
        self.label_6.setText(_translate("AddProtocolMapDialog", "Value:"))

