# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'MapSpecificOptions.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MapSpecificOptions(object):
    def setupUi(self, MapSpecificOptions):
        MapSpecificOptions.setObjectName("MapSpecificOptions")
        MapSpecificOptions.resize(648, 590)
        self.gridLayout = QtWidgets.QGridLayout(MapSpecificOptions)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setHorizontalSpacing(0)
        self.gridLayout.setVerticalSpacing(10)
        self.gridLayout.setObjectName("gridLayout")
        self.frame_3 = CollapsablePanel(MapSpecificOptions)
        self.frame_3.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_3.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame_3.setObjectName("frame_3")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.frame_3)
        self.verticalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_7.setSpacing(0)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.label_22 = CollapsablePanelHeader(self.frame_3)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_22.setFont(font)
        self.label_22.setObjectName("label_22")
        self.verticalLayout_7.addWidget(self.label_22)
        self.line_5 = QtWidgets.QFrame(self.frame_3)
        self.line_5.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_5.setObjectName("line_5")
        self.verticalLayout_7.addWidget(self.line_5)
        self.frame_6 = CollapsablePanelContent(self.frame_3)
        self.frame_6.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_6.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame_6.setObjectName("frame_6")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.frame_6)
        self.gridLayout_6.setContentsMargins(6, 6, 0, 0)
        self.gridLayout_6.setHorizontalSpacing(6)
        self.gridLayout_6.setVerticalSpacing(3)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.gridLayout_7 = QtWidgets.QGridLayout()
        self.gridLayout_7.setContentsMargins(-1, 0, -1, 0)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.label = QtWidgets.QLabel(self.frame_6)
        self.label.setObjectName("label")
        self.gridLayout_7.addWidget(self.label, 0, 0, 1, 1)
        self.use_data_scale_min = QtWidgets.QCheckBox(self.frame_6)
        self.use_data_scale_min.setText("")
        self.use_data_scale_min.setObjectName("use_data_scale_min")
        self.gridLayout_7.addWidget(self.use_data_scale_min, 1, 2, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.frame_6)
        self.label_5.setObjectName("label_5")
        self.gridLayout_7.addWidget(self.label_5, 1, 0, 1, 1)
        self.use_data_scale_max = QtWidgets.QCheckBox(self.frame_6)
        self.use_data_scale_max.setText("")
        self.use_data_scale_max.setObjectName("use_data_scale_max")
        self.gridLayout_7.addWidget(self.use_data_scale_max, 0, 2, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.frame_6)
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_4.setObjectName("label_4")
        self.gridLayout_7.addWidget(self.label_4, 2, 1, 1, 1)
        self.data_set_use_scale = QtWidgets.QCheckBox(self.frame_6)
        self.data_set_use_scale.setText("")
        self.data_set_use_scale.setObjectName("data_set_use_scale")
        self.gridLayout_7.addWidget(self.data_set_use_scale, 2, 2, 1, 1)
        self.data_scale_max = ScientificNumberScroller(self.frame_6)
        self.data_scale_max.setObjectName("data_scale_max")
        self.gridLayout_7.addWidget(self.data_scale_max, 0, 1, 1, 1)
        self.data_scale_min = ScientificNumberScroller(self.frame_6)
        self.data_scale_min.setObjectName("data_scale_min")
        self.gridLayout_7.addWidget(self.data_scale_min, 1, 1, 1, 1)
        self.gridLayout_6.addLayout(self.gridLayout_7, 0, 0, 1, 1)
        self.verticalLayout_7.addWidget(self.frame_6)
        self.gridLayout.addWidget(self.frame_3, 1, 0, 1, 1)
        self.frame = CollapsablePanel(MapSpecificOptions)
        self.frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame.setObjectName("frame")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setSpacing(0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.label_18 = CollapsablePanelHeader(self.frame)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_18.setFont(font)
        self.label_18.setObjectName("label_18")
        self.verticalLayout_5.addWidget(self.label_18)
        self.line_3 = QtWidgets.QFrame(self.frame)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.verticalLayout_5.addWidget(self.line_3)
        self.frame_4 = CollapsablePanelContent(self.frame)
        self.frame_4.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_4.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame_4.setObjectName("frame_4")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.frame_4)
        self.gridLayout_3.setContentsMargins(6, 6, 0, 0)
        self.gridLayout_3.setHorizontalSpacing(6)
        self.gridLayout_3.setVerticalSpacing(3)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label_3 = QtWidgets.QLabel(self.frame_4)
        self.label_3.setObjectName("label_3")
        self.gridLayout_3.addWidget(self.label_3, 2, 0, 1, 1)
        self.colormap = QtWidgets.QComboBox(self.frame_4)
        self.colormap.setObjectName("colormap")
        self.gridLayout_3.addWidget(self.colormap, 3, 1, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.frame_4)
        self.label_9.setObjectName("label_9")
        self.gridLayout_3.addWidget(self.label_9, 3, 0, 1, 1)
        self.label_21 = QtWidgets.QLabel(self.frame_4)
        self.label_21.setObjectName("label_21")
        self.gridLayout_3.addWidget(self.label_21, 0, 0, 1, 1)
        self.map_title = QtWidgets.QPlainTextEdit(self.frame_4)
        self.map_title.setObjectName("map_title")
        self.gridLayout_3.addWidget(self.map_title, 0, 1, 1, 1)
        self.data_colorbar_label = QtWidgets.QPlainTextEdit(self.frame_4)
        self.data_colorbar_label.setObjectName("data_colorbar_label")
        self.gridLayout_3.addWidget(self.data_colorbar_label, 2, 1, 1, 1)
        self.verticalLayout_5.addWidget(self.frame_4)
        self.gridLayout.addWidget(self.frame, 0, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem, 4, 0, 1, 1)
        self.frame_2 = CollapsablePanel(MapSpecificOptions)
        self.frame_2.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame_2.setObjectName("frame_2")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.frame_2)
        self.verticalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_6.setSpacing(0)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.label_20 = CollapsablePanelHeader(self.frame_2)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_20.setFont(font)
        self.label_20.setObjectName("label_20")
        self.verticalLayout_6.addWidget(self.label_20)
        self.line_4 = QtWidgets.QFrame(self.frame_2)
        self.line_4.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_4.setObjectName("line_4")
        self.verticalLayout_6.addWidget(self.line_4)
        self.frame_5 = CollapsablePanelContent(self.frame_2)
        self.frame_5.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_5.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame_5.setObjectName("frame_5")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.frame_5)
        self.gridLayout_4.setContentsMargins(6, 6, 0, 0)
        self.gridLayout_4.setHorizontalSpacing(6)
        self.gridLayout_4.setVerticalSpacing(3)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.info_file_location = QtWidgets.QLabel(self.frame_5)
        self.info_file_location.setWordWrap(True)
        self.info_file_location.setObjectName("info_file_location")
        self.gridLayout_4.addWidget(self.info_file_location, 0, 1, 1, 1)
        self.label_11 = QtWidgets.QLabel(self.frame_5)
        self.label_11.setObjectName("label_11")
        self.gridLayout_4.addWidget(self.label_11, 3, 0, 1, 1)
        self.label_24 = QtWidgets.QLabel(self.frame_5)
        self.label_24.setObjectName("label_24")
        self.gridLayout_4.addWidget(self.label_24, 0, 0, 1, 1)
        self.info_minimum = QtWidgets.QLabel(self.frame_5)
        self.info_minimum.setObjectName("info_minimum")
        self.gridLayout_4.addWidget(self.info_minimum, 3, 1, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.frame_5)
        self.label_6.setObjectName("label_6")
        self.gridLayout_4.addWidget(self.label_6, 1, 0, 1, 1)
        self.info_shape = QtWidgets.QLabel(self.frame_5)
        self.info_shape.setObjectName("info_shape")
        self.gridLayout_4.addWidget(self.info_shape, 1, 1, 1, 1)
        self.label_27 = QtWidgets.QLabel(self.frame_5)
        self.label_27.setObjectName("label_27")
        self.gridLayout_4.addWidget(self.label_27, 5, 0, 1, 1)
        self.info_has_nan = QtWidgets.QLabel(self.frame_5)
        self.info_has_nan.setObjectName("info_has_nan")
        self.gridLayout_4.addWidget(self.info_has_nan, 5, 1, 1, 1)
        self.label_26 = QtWidgets.QLabel(self.frame_5)
        self.label_26.setObjectName("label_26")
        self.gridLayout_4.addWidget(self.label_26, 2, 0, 1, 1)
        self.info_maximum = QtWidgets.QLabel(self.frame_5)
        self.info_maximum.setObjectName("info_maximum")
        self.gridLayout_4.addWidget(self.info_maximum, 2, 1, 1, 1)
        self.gridLayout_4.setColumnStretch(1, 1)
        self.verticalLayout_6.addWidget(self.frame_5)
        self.gridLayout.addWidget(self.frame_2, 3, 0, 1, 1)
        self.info_Clipping = CollapsablePanel(MapSpecificOptions)
        self.info_Clipping.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.info_Clipping.setFrameShadow(QtWidgets.QFrame.Plain)
        self.info_Clipping.setObjectName("info_Clipping")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.info_Clipping)
        self.verticalLayout_8.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_8.setSpacing(0)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.label_25 = CollapsablePanelHeader(self.info_Clipping)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_25.setFont(font)
        self.label_25.setObjectName("label_25")
        self.verticalLayout_8.addWidget(self.label_25)
        self.line_6 = QtWidgets.QFrame(self.info_Clipping)
        self.line_6.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_6.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_6.setObjectName("line_6")
        self.verticalLayout_8.addWidget(self.line_6)
        self.frame_8 = CollapsablePanelContent(self.info_Clipping)
        self.frame_8.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_8.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame_8.setObjectName("frame_8")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.frame_8)
        self.gridLayout_8.setContentsMargins(6, 6, 0, 0)
        self.gridLayout_8.setHorizontalSpacing(6)
        self.gridLayout_8.setVerticalSpacing(3)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.gridLayout_9 = QtWidgets.QGridLayout()
        self.gridLayout_9.setContentsMargins(-1, 0, -1, 0)
        self.gridLayout_9.setObjectName("gridLayout_9")
        self.data_clipping_min = ScientificNumberScroller(self.frame_8)
        self.data_clipping_min.setObjectName("data_clipping_min")
        self.gridLayout_9.addWidget(self.data_clipping_min, 1, 1, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.frame_8)
        self.label_7.setObjectName("label_7")
        self.gridLayout_9.addWidget(self.label_7, 0, 0, 1, 1)
        self.data_set_use_clipping = QtWidgets.QCheckBox(self.frame_8)
        self.data_set_use_clipping.setText("")
        self.data_set_use_clipping.setObjectName("data_set_use_clipping")
        self.gridLayout_9.addWidget(self.data_set_use_clipping, 2, 2, 1, 1)
        self.use_data_clipping_max = QtWidgets.QCheckBox(self.frame_8)
        self.use_data_clipping_max.setText("")
        self.use_data_clipping_max.setObjectName("use_data_clipping_max")
        self.gridLayout_9.addWidget(self.use_data_clipping_max, 0, 2, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.frame_8)
        self.label_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_2.setObjectName("label_2")
        self.gridLayout_9.addWidget(self.label_2, 2, 1, 1, 1)
        self.use_data_clipping_min = QtWidgets.QCheckBox(self.frame_8)
        self.use_data_clipping_min.setText("")
        self.use_data_clipping_min.setObjectName("use_data_clipping_min")
        self.gridLayout_9.addWidget(self.use_data_clipping_min, 1, 2, 1, 1)
        self.data_clipping_max = ScientificNumberScroller(self.frame_8)
        self.data_clipping_max.setObjectName("data_clipping_max")
        self.gridLayout_9.addWidget(self.data_clipping_max, 0, 1, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.frame_8)
        self.label_8.setObjectName("label_8")
        self.gridLayout_9.addWidget(self.label_8, 1, 0, 1, 1)
        self.gridLayout_8.addLayout(self.gridLayout_9, 0, 0, 1, 1)
        self.verticalLayout_8.addWidget(self.frame_8)
        self.gridLayout.addWidget(self.info_Clipping, 2, 0, 1, 1)

        self.retranslateUi(MapSpecificOptions)
        QtCore.QMetaObject.connectSlotsByName(MapSpecificOptions)
        MapSpecificOptions.setTabOrder(self.colormap, self.use_data_scale_max)
        MapSpecificOptions.setTabOrder(self.use_data_scale_max, self.use_data_scale_min)
        MapSpecificOptions.setTabOrder(self.use_data_scale_min, self.data_set_use_scale)
        MapSpecificOptions.setTabOrder(self.data_set_use_scale, self.use_data_clipping_max)
        MapSpecificOptions.setTabOrder(self.use_data_clipping_max, self.use_data_clipping_min)
        MapSpecificOptions.setTabOrder(self.use_data_clipping_min, self.data_set_use_clipping)

    def retranslateUi(self, MapSpecificOptions):
        _translate = QtCore.QCoreApplication.translate
        MapSpecificOptions.setWindowTitle(_translate("MapSpecificOptions", "Form"))
        self.label_22.setText(_translate("MapSpecificOptions", "Scale"))
        self.label.setText(_translate("MapSpecificOptions", "Max:"))
        self.label_5.setText(_translate("MapSpecificOptions", "Min:"))
        self.label_4.setText(_translate("MapSpecificOptions", "Enable:"))
        self.label_18.setText(_translate("MapSpecificOptions", "General"))
        self.label_3.setText(_translate("MapSpecificOptions", "Colorbar label:"))
        self.label_9.setText(_translate("MapSpecificOptions", "Colormap:"))
        self.label_21.setText(_translate("MapSpecificOptions", "Title (latex):"))
        self.label_20.setText(_translate("MapSpecificOptions", "Info"))
        self.info_file_location.setText(_translate("MapSpecificOptions", "TextLabel"))
        self.label_11.setText(_translate("MapSpecificOptions", "Minimum:"))
        self.label_24.setText(_translate("MapSpecificOptions", "Filename:"))
        self.info_minimum.setText(_translate("MapSpecificOptions", "TextLabel"))
        self.label_6.setText(_translate("MapSpecificOptions", "Shape:"))
        self.info_shape.setText(_translate("MapSpecificOptions", "TextLabel"))
        self.label_27.setText(_translate("MapSpecificOptions", "Has NaNs:"))
        self.info_has_nan.setText(_translate("MapSpecificOptions", "TextLabel"))
        self.label_26.setText(_translate("MapSpecificOptions", "Maximum:"))
        self.info_maximum.setText(_translate("MapSpecificOptions", "TextLabel"))
        self.label_25.setText(_translate("MapSpecificOptions", "Clipping"))
        self.label_7.setText(_translate("MapSpecificOptions", "Max:"))
        self.label_2.setText(_translate("MapSpecificOptions", "Enable:"))
        self.label_8.setText(_translate("MapSpecificOptions", "Min:"))

from ..widgets import CollapsablePanel, CollapsablePanelContent, CollapsablePanelHeader
from mdt.gui.widgets.composite_widgets import ScientificNumberScroller
