# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'TabGeneral.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_TabGeneral(object):
    def setupUi(self, TabGeneral):
        TabGeneral.setObjectName("TabGeneral")
        TabGeneral.resize(963, 704)
        self.gridLayout = QtWidgets.QGridLayout(TabGeneral)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.scrollArea = QtWidgets.QScrollArea(TabGeneral)
        self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, -289, 947, 1075))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_6.setContentsMargins(6, 6, 6, 6)
        self.gridLayout_6.setHorizontalSpacing(0)
        self.gridLayout_6.setVerticalSpacing(10)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.frame = CollapsablePanel(self.scrollAreaWidgetContents)
        self.frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame.setObjectName("frame")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setSpacing(0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.label_13 = CollapsablePanelHeader(self.frame)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_13.setFont(font)
        self.label_13.setObjectName("label_13")
        self.verticalLayout_4.addWidget(self.label_13)
        self.line_2 = QtWidgets.QFrame(self.frame)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_4.addWidget(self.line_2)
        self.frame_3 = CollapsablePanelContent(self.frame)
        self.frame_3.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_3.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame_3.setObjectName("frame_3")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.frame_3)
        self.gridLayout_2.setContentsMargins(6, 6, 0, 0)
        self.gridLayout_2.setHorizontalSpacing(6)
        self.gridLayout_2.setVerticalSpacing(3)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setContentsMargins(3, -1, -1, -1)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_10 = QtWidgets.QLabel(self.frame_3)
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_7.addWidget(self.label_10)
        self.maximumDimension = QtWidgets.QLabel(self.frame_3)
        self.maximumDimension.setObjectName("maximumDimension")
        self.horizontalLayout_7.addWidget(self.maximumDimension)
        self.gridLayout_2.addLayout(self.horizontalLayout_7, 0, 2, 1, 1)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setContentsMargins(3, -1, -1, -1)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_9 = QtWidgets.QLabel(self.frame_3)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_6.addWidget(self.label_9)
        self.maximumIndex = QtWidgets.QLabel(self.frame_3)
        self.maximumIndex.setObjectName("maximumIndex")
        self.horizontalLayout_6.addWidget(self.maximumIndex)
        self.gridLayout_2.addLayout(self.horizontalLayout_6, 1, 2, 1, 1)
        self.label_14 = QtWidgets.QLabel(self.frame_3)
        self.label_14.setObjectName("label_14")
        self.gridLayout_2.addWidget(self.label_14, 0, 0, 1, 1)
        self.general_dimension = QtWidgets.QSpinBox(self.frame_3)
        self.general_dimension.setObjectName("general_dimension")
        self.gridLayout_2.addWidget(self.general_dimension, 0, 1, 1, 1)
        self.label_16 = QtWidgets.QLabel(self.frame_3)
        self.label_16.setObjectName("label_16")
        self.gridLayout_2.addWidget(self.label_16, 1, 0, 1, 1)
        self.general_slice_index = QtWidgets.QSpinBox(self.frame_3)
        self.general_slice_index.setObjectName("general_slice_index")
        self.gridLayout_2.addWidget(self.general_slice_index, 1, 1, 1, 1)
        self.label_15 = QtWidgets.QLabel(self.frame_3)
        self.label_15.setObjectName("label_15")
        self.gridLayout_2.addWidget(self.label_15, 2, 0, 1, 1)
        self.general_volume_index = QtWidgets.QSpinBox(self.frame_3)
        self.general_volume_index.setObjectName("general_volume_index")
        self.gridLayout_2.addWidget(self.general_volume_index, 2, 1, 1, 1)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setContentsMargins(3, -1, -1, -1)
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.label_25 = QtWidgets.QLabel(self.frame_3)
        self.label_25.setObjectName("label_25")
        self.horizontalLayout_10.addWidget(self.label_25)
        self.maximumVolume = QtWidgets.QLabel(self.frame_3)
        self.maximumVolume.setObjectName("maximumVolume")
        self.horizontalLayout_10.addWidget(self.maximumVolume)
        self.gridLayout_2.addLayout(self.horizontalLayout_10, 2, 2, 1, 1)
        self.gridLayout_2.setColumnStretch(1, 1)
        self.verticalLayout_4.addWidget(self.frame_3)
        self.gridLayout_6.addWidget(self.frame, 0, 0, 1, 1)
        self.general_Miscellaneous = CollapsablePanel(self.scrollAreaWidgetContents)
        self.general_Miscellaneous.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.general_Miscellaneous.setFrameShadow(QtWidgets.QFrame.Plain)
        self.general_Miscellaneous.setObjectName("general_Miscellaneous")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.general_Miscellaneous)
        self.verticalLayout_8.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_8.setSpacing(0)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.label_23 = CollapsablePanelHeader(self.general_Miscellaneous)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_23.setFont(font)
        self.label_23.setObjectName("label_23")
        self.verticalLayout_8.addWidget(self.label_23)
        self.line_6 = QtWidgets.QFrame(self.general_Miscellaneous)
        self.line_6.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_6.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_6.setObjectName("line_6")
        self.verticalLayout_8.addWidget(self.line_6)
        self.frame_10 = CollapsablePanelContent(self.general_Miscellaneous)
        self.frame_10.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_10.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame_10.setObjectName("frame_10")
        self.formLayout = QtWidgets.QFormLayout(self.frame_10)
        self.formLayout.setContentsMargins(6, 6, 0, 0)
        self.formLayout.setHorizontalSpacing(6)
        self.formLayout.setVerticalSpacing(3)
        self.formLayout.setObjectName("formLayout")
        self.label = QtWidgets.QLabel(self.frame_10)
        self.label.setObjectName("label")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label)
        self.general_colormap = QtWidgets.QComboBox(self.frame_10)
        self.general_colormap.setObjectName("general_colormap")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.general_colormap)
        self.label_2 = QtWidgets.QLabel(self.frame_10)
        self.label_2.setObjectName("label_2")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_2)
        self.general_rotate = QtWidgets.QComboBox(self.frame_10)
        self.general_rotate.setObjectName("general_rotate")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.general_rotate)
        self.label_4 = QtWidgets.QLabel(self.frame_10)
        self.label_4.setObjectName("label_4")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.LabelRole, self.label_4)
        self.general_show_axis = QtWidgets.QCheckBox(self.frame_10)
        self.general_show_axis.setText("")
        self.general_show_axis.setObjectName("general_show_axis")
        self.formLayout.setWidget(4, QtWidgets.QFormLayout.FieldRole, self.general_show_axis)
        self.label_11 = QtWidgets.QLabel(self.frame_10)
        self.label_11.setObjectName("label_11")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.LabelRole, self.label_11)
        self.general_flipud = QtWidgets.QCheckBox(self.frame_10)
        self.general_flipud.setText("")
        self.general_flipud.setObjectName("general_flipud")
        self.formLayout.setWidget(3, QtWidgets.QFormLayout.FieldRole, self.general_flipud)
        self.label_27 = QtWidgets.QLabel(self.frame_10)
        self.label_27.setObjectName("label_27")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.LabelRole, self.label_27)
        self.general_interpolation = QtWidgets.QComboBox(self.frame_10)
        self.general_interpolation.setObjectName("general_interpolation")
        self.formLayout.setWidget(7, QtWidgets.QFormLayout.FieldRole, self.general_interpolation)
        self.label_28 = QtWidgets.QLabel(self.frame_10)
        self.label_28.setObjectName("label_28")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_28)
        self.plot_title = QtWidgets.QLineEdit(self.frame_10)
        self.plot_title.setObjectName("plot_title")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.plot_title)
        self.label_29 = QtWidgets.QLabel(self.frame_10)
        self.label_29.setObjectName("label_29")
        self.formLayout.setWidget(8, QtWidgets.QFormLayout.LabelRole, self.label_29)
        self.mask_name = QtWidgets.QComboBox(self.frame_10)
        self.mask_name.setObjectName("mask_name")
        self.formLayout.setWidget(8, QtWidgets.QFormLayout.FieldRole, self.mask_name)
        self.label_33 = QtWidgets.QLabel(self.frame_10)
        self.label_33.setObjectName("label_33")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.LabelRole, self.label_33)
        self.general_show_plot_titles = QtWidgets.QCheckBox(self.frame_10)
        self.general_show_plot_titles.setText("")
        self.general_show_plot_titles.setObjectName("general_show_plot_titles")
        self.formLayout.setWidget(6, QtWidgets.QFormLayout.FieldRole, self.general_show_plot_titles)
        self.verticalLayout_8.addWidget(self.frame_10)
        self.gridLayout_6.addWidget(self.general_Miscellaneous, 8, 0, 1, 1)
        self.general_DisplayOrder = CollapsablePanel(self.scrollAreaWidgetContents)
        self.general_DisplayOrder.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.general_DisplayOrder.setFrameShadow(QtWidgets.QFrame.Plain)
        self.general_DisplayOrder.setObjectName("general_DisplayOrder")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.general_DisplayOrder)
        self.verticalLayout_7.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_7.setSpacing(0)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.label_22 = CollapsablePanelHeader(self.general_DisplayOrder)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_22.setFont(font)
        self.label_22.setObjectName("label_22")
        self.verticalLayout_7.addWidget(self.label_22)
        self.line_5 = QtWidgets.QFrame(self.general_DisplayOrder)
        self.line_5.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_5.setObjectName("line_5")
        self.verticalLayout_7.addWidget(self.line_5)
        self.frame_9 = CollapsablePanelContent(self.general_DisplayOrder)
        self.frame_9.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_9.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame_9.setObjectName("frame_9")
        self.gridLayout_11 = QtWidgets.QGridLayout(self.frame_9)
        self.gridLayout_11.setContentsMargins(6, 6, 0, 0)
        self.gridLayout_11.setHorizontalSpacing(6)
        self.gridLayout_11.setVerticalSpacing(3)
        self.gridLayout_11.setObjectName("gridLayout_11")
        self.general_display_order = MapsReorderer(self.frame_9)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.general_display_order.sizePolicy().hasHeightForWidth())
        self.general_display_order.setSizePolicy(sizePolicy)
        self.general_display_order.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.general_display_order.setObjectName("general_display_order")
        self.gridLayout_11.addWidget(self.general_display_order, 0, 0, 1, 1)
        self.verticalLayout_7.addWidget(self.frame_9)
        self.gridLayout_6.addWidget(self.general_DisplayOrder, 4, 0, 1, 1)
        self.frame_6 = CollapsablePanel(self.scrollAreaWidgetContents)
        self.frame_6.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_6.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame_6.setObjectName("frame_6")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.frame_6)
        self.verticalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_6.setSpacing(0)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.label_21 = CollapsablePanelHeader(self.frame_6)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_21.setFont(font)
        self.label_21.setObjectName("label_21")
        self.verticalLayout_6.addWidget(self.label_21)
        self.line_4 = QtWidgets.QFrame(self.frame_6)
        self.line_4.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_4.setObjectName("line_4")
        self.verticalLayout_6.addWidget(self.line_4)
        self.frame_7 = CollapsablePanelContent(self.frame_6)
        self.frame_7.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_7.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame_7.setObjectName("frame_7")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.frame_7)
        self.gridLayout_5.setContentsMargins(6, 6, 0, 0)
        self.gridLayout_5.setHorizontalSpacing(6)
        self.gridLayout_5.setVerticalSpacing(3)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.general_map_selection = QtWidgets.QListWidget(self.frame_7)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.general_map_selection.sizePolicy().hasHeightForWidth())
        self.general_map_selection.setSizePolicy(sizePolicy)
        self.general_map_selection.setSelectionMode(QtWidgets.QAbstractItemView.MultiSelection)
        self.general_map_selection.setObjectName("general_map_selection")
        self.gridLayout_5.addWidget(self.general_map_selection, 0, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(-1, -1, -1, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.general_deselect_all_maps = QtWidgets.QPushButton(self.frame_7)
        self.general_deselect_all_maps.setObjectName("general_deselect_all_maps")
        self.horizontalLayout.addWidget(self.general_deselect_all_maps)
        self.general_invert_map_selection = QtWidgets.QPushButton(self.frame_7)
        self.general_invert_map_selection.setObjectName("general_invert_map_selection")
        self.horizontalLayout.addWidget(self.general_invert_map_selection)
        self.gridLayout_5.addLayout(self.horizontalLayout, 1, 0, 1, 1)
        self.verticalLayout_6.addWidget(self.frame_7)
        self.gridLayout_6.addWidget(self.frame_6, 2, 0, 1, 1)
        self.general_Info = CollapsablePanel(self.scrollAreaWidgetContents)
        self.general_Info.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.general_Info.setFrameShadow(QtWidgets.QFrame.Plain)
        self.general_Info.setObjectName("general_Info")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.general_Info)
        self.verticalLayout_9.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_9.setSpacing(0)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.label_24 = CollapsablePanelHeader(self.general_Info)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_24.setFont(font)
        self.label_24.setObjectName("label_24")
        self.verticalLayout_9.addWidget(self.label_24)
        self.line_7 = QtWidgets.QFrame(self.general_Info)
        self.line_7.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_7.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_7.setObjectName("line_7")
        self.verticalLayout_9.addWidget(self.line_7)
        self.frame_8 = CollapsablePanelContent(self.general_Info)
        self.frame_8.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_8.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame_8.setObjectName("frame_8")
        self.gridLayout_12 = QtWidgets.QGridLayout(self.frame_8)
        self.gridLayout_12.setContentsMargins(6, 6, 0, 0)
        self.gridLayout_12.setHorizontalSpacing(6)
        self.gridLayout_12.setVerticalSpacing(3)
        self.gridLayout_12.setObjectName("gridLayout_12")
        self.label_7 = QtWidgets.QLabel(self.frame_8)
        self.label_7.setObjectName("label_7")
        self.gridLayout_12.addWidget(self.label_7, 1, 0, 1, 1)
        self.general_info_nmr_maps = QtWidgets.QLabel(self.frame_8)
        self.general_info_nmr_maps.setObjectName("general_info_nmr_maps")
        self.gridLayout_12.addWidget(self.general_info_nmr_maps, 1, 1, 1, 1)
        self.verticalLayout_9.addWidget(self.frame_8)
        self.gridLayout_6.addWidget(self.general_Info, 10, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_6.addItem(spacerItem, 11, 0, 1, 1)
        self.general_Font = CollapsablePanel(self.scrollAreaWidgetContents)
        self.general_Font.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.general_Font.setFrameShadow(QtWidgets.QFrame.Plain)
        self.general_Font.setObjectName("general_Font")
        self.verticalLayout_10 = QtWidgets.QVBoxLayout(self.general_Font)
        self.verticalLayout_10.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_10.setSpacing(0)
        self.verticalLayout_10.setObjectName("verticalLayout_10")
        self.label_26 = CollapsablePanelHeader(self.general_Font)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_26.setFont(font)
        self.label_26.setObjectName("label_26")
        self.verticalLayout_10.addWidget(self.label_26)
        self.line_8 = QtWidgets.QFrame(self.general_Font)
        self.line_8.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_8.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_8.setObjectName("line_8")
        self.verticalLayout_10.addWidget(self.line_8)
        self.frame_11 = CollapsablePanelContent(self.general_Font)
        self.frame_11.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_11.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame_11.setObjectName("frame_11")
        self.formLayout_2 = QtWidgets.QFormLayout(self.frame_11)
        self.formLayout_2.setContentsMargins(6, 6, 0, 0)
        self.formLayout_2.setHorizontalSpacing(6)
        self.formLayout_2.setVerticalSpacing(3)
        self.formLayout_2.setObjectName("formLayout_2")
        self.label_6 = QtWidgets.QLabel(self.frame_11)
        self.label_6.setObjectName("label_6")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_6)
        self.general_font_family = QtWidgets.QComboBox(self.frame_11)
        self.general_font_family.setObjectName("general_font_family")
        self.formLayout_2.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.general_font_family)
        self.label_8 = QtWidgets.QLabel(self.frame_11)
        self.label_8.setObjectName("label_8")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_8)
        self.general_font_size = QtWidgets.QSpinBox(self.frame_11)
        self.general_font_size.setMinimum(1)
        self.general_font_size.setObjectName("general_font_size")
        self.formLayout_2.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.general_font_size)
        self.verticalLayout_10.addWidget(self.frame_11)
        self.gridLayout_6.addWidget(self.general_Font, 6, 0, 1, 1)
        self.general_Zoom = CollapsablePanel(self.scrollAreaWidgetContents)
        self.general_Zoom.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.general_Zoom.setFrameShadow(QtWidgets.QFrame.Plain)
        self.general_Zoom.setObjectName("general_Zoom")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.general_Zoom)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setSpacing(0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.label_17 = CollapsablePanelHeader(self.general_Zoom)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_17.setFont(font)
        self.label_17.setObjectName("label_17")
        self.verticalLayout_5.addWidget(self.label_17)
        self.line_3 = QtWidgets.QFrame(self.general_Zoom)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.verticalLayout_5.addWidget(self.line_3)
        self.frame_5 = CollapsablePanelContent(self.general_Zoom)
        self.frame_5.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_5.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame_5.setObjectName("frame_5")
        self.gridLayout_10 = QtWidgets.QGridLayout(self.frame_5)
        self.gridLayout_10.setContentsMargins(6, 6, 0, 0)
        self.gridLayout_10.setHorizontalSpacing(6)
        self.gridLayout_10.setVerticalSpacing(3)
        self.gridLayout_10.setObjectName("gridLayout_10")
        self.general_zoom_x_1 = QtWidgets.QSpinBox(self.frame_5)
        self.general_zoom_x_1.setObjectName("general_zoom_x_1")
        self.gridLayout_10.addWidget(self.general_zoom_x_1, 1, 3, 1, 1)
        self.label_19 = QtWidgets.QLabel(self.frame_5)
        self.label_19.setObjectName("label_19")
        self.gridLayout_10.addWidget(self.label_19, 7, 0, 1, 1)
        self.label_20 = QtWidgets.QLabel(self.frame_5)
        self.label_20.setObjectName("label_20")
        self.gridLayout_10.addWidget(self.label_20, 1, 0, 1, 1)
        self.general_zoom_x_0 = QtWidgets.QSpinBox(self.frame_5)
        self.general_zoom_x_0.setObjectName("general_zoom_x_0")
        self.gridLayout_10.addWidget(self.general_zoom_x_0, 1, 1, 1, 1)
        self.general_zoom_y_1 = QtWidgets.QSpinBox(self.frame_5)
        self.general_zoom_y_1.setObjectName("general_zoom_y_1")
        self.gridLayout_10.addWidget(self.general_zoom_y_1, 7, 3, 1, 1)
        self.label_18 = QtWidgets.QLabel(self.frame_5)
        self.label_18.setObjectName("label_18")
        self.gridLayout_10.addWidget(self.label_18, 7, 2, 1, 1)
        self.general_zoom_y_0 = QtWidgets.QSpinBox(self.frame_5)
        self.general_zoom_y_0.setObjectName("general_zoom_y_0")
        self.gridLayout_10.addWidget(self.general_zoom_y_0, 7, 1, 1, 1)
        self.label_12 = QtWidgets.QLabel(self.frame_5)
        self.label_12.setObjectName("label_12")
        self.gridLayout_10.addWidget(self.label_12, 1, 2, 1, 1)
        self.general_zoom_fit = QtWidgets.QPushButton(self.frame_5)
        self.general_zoom_fit.setObjectName("general_zoom_fit")
        self.gridLayout_10.addWidget(self.general_zoom_fit, 8, 1, 1, 1)
        self.general_zoom_reset = QtWidgets.QPushButton(self.frame_5)
        self.general_zoom_reset.setObjectName("general_zoom_reset")
        self.gridLayout_10.addWidget(self.general_zoom_reset, 8, 3, 1, 1)
        self.gridLayout_10.setColumnStretch(1, 1)
        self.gridLayout_10.setColumnStretch(3, 1)
        self.verticalLayout_5.addWidget(self.frame_5)
        self.gridLayout_6.addWidget(self.general_Zoom, 5, 0, 1, 1)
        self.general_Colorbar = CollapsablePanel(self.scrollAreaWidgetContents)
        self.general_Colorbar.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.general_Colorbar.setFrameShadow(QtWidgets.QFrame.Plain)
        self.general_Colorbar.setObjectName("general_Colorbar")
        self.verticalLayout_11 = QtWidgets.QVBoxLayout(self.general_Colorbar)
        self.verticalLayout_11.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_11.setSpacing(0)
        self.verticalLayout_11.setObjectName("verticalLayout_11")
        self.label_30 = CollapsablePanelHeader(self.general_Colorbar)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.label_30.setFont(font)
        self.label_30.setObjectName("label_30")
        self.verticalLayout_11.addWidget(self.label_30)
        self.line_9 = QtWidgets.QFrame(self.general_Colorbar)
        self.line_9.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_9.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_9.setObjectName("line_9")
        self.verticalLayout_11.addWidget(self.line_9)
        self.frame_12 = CollapsablePanelContent(self.general_Colorbar)
        self.frame_12.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame_12.setFrameShadow(QtWidgets.QFrame.Plain)
        self.frame_12.setObjectName("frame_12")
        self.formLayout_3 = QtWidgets.QFormLayout(self.frame_12)
        self.formLayout_3.setContentsMargins(6, 6, 0, 0)
        self.formLayout_3.setHorizontalSpacing(6)
        self.formLayout_3.setVerticalSpacing(3)
        self.formLayout_3.setObjectName("formLayout_3")
        self.label_5 = QtWidgets.QLabel(self.frame_12)
        self.label_5.setObjectName("label_5")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.label_5)
        self.general_colorbar_nmr_ticks = QtWidgets.QSpinBox(self.frame_12)
        self.general_colorbar_nmr_ticks.setMinimum(2)
        self.general_colorbar_nmr_ticks.setProperty("value", 10)
        self.general_colorbar_nmr_ticks.setObjectName("general_colorbar_nmr_ticks")
        self.formLayout_3.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.general_colorbar_nmr_ticks)
        self.label_31 = QtWidgets.QLabel(self.frame_12)
        self.label_31.setObjectName("label_31")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.label_31)
        self.general_show_colorbar = QtWidgets.QCheckBox(self.frame_12)
        self.general_show_colorbar.setText("")
        self.general_show_colorbar.setObjectName("general_show_colorbar")
        self.formLayout_3.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.general_show_colorbar)
        self.label_32 = QtWidgets.QLabel(self.frame_12)
        self.label_32.setObjectName("label_32")
        self.formLayout_3.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.label_32)
        self.general_colorbar_location = QtWidgets.QComboBox(self.frame_12)
        self.general_colorbar_location.setObjectName("general_colorbar_location")
        self.general_colorbar_location.addItem("")
        self.general_colorbar_location.addItem("")
        self.general_colorbar_location.addItem("")
        self.general_colorbar_location.addItem("")
        self.formLayout_3.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.general_colorbar_location)
        self.verticalLayout_11.addWidget(self.frame_12)
        self.gridLayout_6.addWidget(self.general_Colorbar, 7, 0, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 0, 0, 1, 1)

        self.retranslateUi(TabGeneral)
        QtCore.QMetaObject.connectSlotsByName(TabGeneral)
        TabGeneral.setTabOrder(self.scrollArea, self.general_dimension)
        TabGeneral.setTabOrder(self.general_dimension, self.general_slice_index)
        TabGeneral.setTabOrder(self.general_slice_index, self.general_volume_index)
        TabGeneral.setTabOrder(self.general_volume_index, self.general_zoom_x_0)
        TabGeneral.setTabOrder(self.general_zoom_x_0, self.general_zoom_y_0)
        TabGeneral.setTabOrder(self.general_zoom_y_0, self.general_zoom_x_1)
        TabGeneral.setTabOrder(self.general_zoom_x_1, self.general_zoom_y_1)
        TabGeneral.setTabOrder(self.general_zoom_y_1, self.general_map_selection)
        TabGeneral.setTabOrder(self.general_map_selection, self.general_display_order)
        TabGeneral.setTabOrder(self.general_display_order, self.general_colormap)
        TabGeneral.setTabOrder(self.general_colormap, self.general_rotate)
        TabGeneral.setTabOrder(self.general_rotate, self.general_show_axis)

    def retranslateUi(self, TabGeneral):
        _translate = QtCore.QCoreApplication.translate
        TabGeneral.setWindowTitle(_translate("TabGeneral", "Form"))
        self.label_13.setText(_translate("TabGeneral", "Index"))
        self.label_10.setText(_translate("TabGeneral", "/ "))
        self.maximumDimension.setText(_translate("TabGeneral", "x"))
        self.label_9.setText(_translate("TabGeneral", "/ "))
        self.maximumIndex.setText(_translate("TabGeneral", "x"))
        self.label_14.setText(_translate("TabGeneral", "Dimension:"))
        self.label_16.setText(_translate("TabGeneral", "Slice index:"))
        self.label_15.setText(_translate("TabGeneral", "Volume:"))
        self.label_25.setText(_translate("TabGeneral", "/ "))
        self.maximumVolume.setText(_translate("TabGeneral", "x"))
        self.label_23.setText(_translate("TabGeneral", "Miscellaneous"))
        self.label.setText(_translate("TabGeneral", "Colormap:"))
        self.label_2.setText(_translate("TabGeneral", "Rotate:"))
        self.label_4.setText(_translate("TabGeneral", "Show axis:"))
        self.label_11.setText(_translate("TabGeneral", "Flip up/down:"))
        self.label_27.setText(_translate("TabGeneral", "Interpolation:"))
        self.label_28.setText(_translate("TabGeneral", "Plot title:"))
        self.label_29.setText(_translate("TabGeneral", "Mask:"))
        self.label_33.setText(_translate("TabGeneral", "Show plot titles:"))
        self.label_22.setText(_translate("TabGeneral", "Display order"))
        self.label_21.setText(_translate("TabGeneral", "Map selection"))
        self.general_deselect_all_maps.setText(_translate("TabGeneral", "Deselect all"))
        self.general_invert_map_selection.setText(_translate("TabGeneral", "Invert selection"))
        self.label_24.setText(_translate("TabGeneral", "Info"))
        self.label_7.setText(_translate("TabGeneral", "Map count:"))
        self.general_info_nmr_maps.setText(_translate("TabGeneral", "-"))
        self.label_26.setText(_translate("TabGeneral", "Font"))
        self.label_6.setText(_translate("TabGeneral", "Family:"))
        self.label_8.setText(_translate("TabGeneral", "Size:"))
        self.label_17.setText(_translate("TabGeneral", "Zoom"))
        self.label_19.setText(_translate("TabGeneral", "y<sub>0</sub>:"))
        self.label_20.setText(_translate("TabGeneral", "<html><head/><body><p>x<span style=\" vertical-align:sub;\">0</span>:</p></body></html>"))
        self.label_18.setText(_translate("TabGeneral", "y<sub>1</sub>:"))
        self.label_12.setText(_translate("TabGeneral", "x<sub>1</sub>:"))
        self.general_zoom_fit.setText(_translate("TabGeneral", "Zoom fit"))
        self.general_zoom_reset.setText(_translate("TabGeneral", "Reset"))
        self.label_30.setText(_translate("TabGeneral", "Colorbar"))
        self.label_5.setText(_translate("TabGeneral", "Number of ticks:"))
        self.label_31.setText(_translate("TabGeneral", "Show colorbar:"))
        self.label_32.setText(_translate("TabGeneral", "Location:"))
        self.general_colorbar_location.setItemText(0, _translate("TabGeneral", "Right"))
        self.general_colorbar_location.setItemText(1, _translate("TabGeneral", "Bottom"))
        self.general_colorbar_location.setItemText(2, _translate("TabGeneral", "Left"))
        self.general_colorbar_location.setItemText(3, _translate("TabGeneral", "Top"))

from ..widgets import CollapsablePanel, CollapsablePanelContent, CollapsablePanelHeader, MapsReorderer
